/*
 * Copyright (c) 2018-2019. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.img.board;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.imageio.ImageIO;

import org.hedgecode.chess.img.AbstractImageLoader;
import org.hedgecode.chess.img.ImageConstants;
import org.hedgecode.chess.img.ImageException;
import org.hedgecode.chess.img.ImageFilter;

/**
 *
 *
 * @author Dmitry Samoshin aka gotty
 */
public class BoardLoader extends AbstractImageLoader {

    private static final File SQUARES_PATH = new File(IMAGES_PATH, ImageConstants.RESOURCE_SQUARES_DIR);

    private static final String SQUARE_DARK = ImageConstants.DARK_SQUARE_FILENAME;
    private static final String SQUARE_LIGHT = ImageConstants.LIGHT_SQUARE_FILENAME;

    private static final String[] SQUARE_NAMES = {SQUARE_DARK, SQUARE_LIGHT};

    private Map<String, Board> squareSetMap = new HashMap<>();

    @Override
    public Board load(String name) throws ImageException {
        Board board = null;
        SquarePair<BufferedImage> squarePair = null;
        if (Type.STATELESS.equals(loadType()) || !squareSetMap.containsKey(name)) {
            File boardPath = new File(SQUARES_PATH, name);
            if (boardPath.exists() && boardPath.isDirectory()) {
                squarePair = loadSquares(name, boardPath);
            }
            if (squarePair != null) {
                board = Board.create(squarePair);
                if (Type.STATEFUL.equals(loadType())) {
                    squareSetMap.put(name, board);
                }
            }
        } else {
            board = squareSetMap.get(name);
        }
        return board;
    }

    @Override
    public void unload(String name) {
        if (Type.STATEFUL.equals(loadType()) && squareSetMap.containsKey(name)) {
            squareSetMap.remove(name);
        }
    }

    @Override
    protected void clear() {
        squareSetMap.clear();
    }

    private SquarePair<BufferedImage> loadSquares(String name, File boardPath) throws ImageException {
        BufferedImage dark = null, light = null;
        File[] images = boardPath.listFiles(new ImageFilter(SQUARE_NAMES));
        if (images != null) {
            for (File image : images) {
                if (image.isFile()) {
                    String filename = image.getName();
                    try {
                        switch (filename.substring(0, filename.lastIndexOf('.'))) {
                            case SQUARE_DARK:
                                dark = ImageIO.read(image);
                                break;
                            case SQUARE_LIGHT:
                                light = ImageIO.read(image);
                                break;
                        }
                    } catch (IOException e) {
                        throw new ImageException(e.getMessage());  // todo: locale
                    }
                }
            }
        }
        return (dark != null && light != null) ? SquarePair.create(dark, light) : null;
    }

}
