/*
 * Copyright (c) 2018. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.uci;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.List;
import java.util.Scanner;

import org.hedgecode.chess.uci.command.OptionParams;
import org.hedgecode.chess.uci.command.OptionType;

/**
 *
 *
 * @author Dmitry Samoshin aka gotty
 */
public class ConsoleAcceptor implements Acceptor {

    private class ConsoleTransmitter implements Transmitter<String> {

        private Scanner scanner;
        private boolean isActive;

        ConsoleTransmitter(InputStream input) {
            scanner = new Scanner(input);
            isActive = true;
        }

        @Override
        public boolean isActive() {
            return isActive;
        }

        @Override
        public boolean hasCommand() {
            return isActive && scanner.hasNextLine();
        }

        @Override
        public String transmitCommand() {
            return scanner.nextLine();
        }

        @Override
        public void addCommand(String command) {
            // not supported for console acceptor
        }

        @Override
        public void stop() {
            isActive = false;
        }

        @Override
        public void close() {
            scanner.close();
        }
    }

    private ConsoleTransmitter transmitter;
    private PrintWriter writer;

    ConsoleAcceptor(InputStream input, OutputStream output) {
        transmitter = new ConsoleTransmitter(input);
        writer = new PrintWriter(output, true);
    }

    public Transmitter<String> transmitter() {
        return transmitter;
    }

    @Override
    public void start() {
    }

    @Override
    public void id(String params) {
        console(UCIConstants.ID, params);
    }

    @Override
    public void uciOK(String params) {
        console(UCIConstants.UCI_OK, params);
    }

    @Override
    public void readyOK(String params) {
        console(UCIConstants.READY_OK, params);
    }

    @Override
    public void setOption(String name) {
        console(
                UCIConstants.OPTION,
                String.format(
                        "%s %s %s %s",
                        OptionParams.OPTION_NAME, name, OptionParams.OPTION_TYPE, OptionType.BUTTON.type()
                )
        );
    }

    @Override
    public void setOption(String name, boolean def) {
        console(
                UCIConstants.OPTION,
                String.format(
                        "%s %s %s %s %s %b",
                        OptionParams.OPTION_NAME, name, OptionParams.OPTION_TYPE, OptionType.CHECK.type(),
                        OptionParams.OPTION_DEFAULT, def
                )
        );
    }

    @Override
    public void setOption(String name, int def, int min, int max) {
        console(
                UCIConstants.OPTION,
                String.format(
                        "%s %s %s %s %s %d %s %d %s %d",
                        OptionParams.OPTION_NAME, name, OptionParams.OPTION_TYPE, OptionType.SPIN.type(),
                        OptionParams.OPTION_DEFAULT, def, OptionParams.OPTION_MIN, min, OptionParams.OPTION_MAX, max
                )
        );
    }

    @Override
    public void setOption(String name, String def) {
        console(
                UCIConstants.OPTION,
                String.format(
                        "%s %s %s %s %s %s",
                        OptionParams.OPTION_NAME, name, OptionParams.OPTION_TYPE, OptionType.STRING.type(),
                        OptionParams.OPTION_DEFAULT, def == null ? "" : def
                )
        );
    }

    @Override
    public void setOption(String name, String def, List<String> vars) {
        StringBuilder varOptions = new StringBuilder();
        for (String var : vars)
            varOptions.append(
                    String.format("%s %s ", OptionParams.OPTION_VAR, var == null ? "" : var)
            );
        console(
                UCIConstants.OPTION,
                String.format(
                        "%s %s %s %s %s %s %s",
                        OptionParams.OPTION_NAME, name, OptionParams.OPTION_TYPE, OptionType.COMBO.type(),
                        OptionParams.OPTION_DEFAULT, def == null ? "" : def, varOptions.toString().trim()
                )
        );
    }

    @Override
    public void terminate(String params) {
        transmitter.stop();
    }

    private void console(String name, String params) {
        writer.println(
                String.format(
                        "%s %s",
                        name,
                        params != null ? params : ""
                ).trim()
        );
    }

}
